@@ -8,7 +8,11 @@ of ``sc.sc_parallel.parallelize()``.
 
 # Import everything
 from .sc_version   import *
+from .sc_settings  import *
 from .sc_utils     import *
+from .sc_printing  import *
+from .sc_datetime  import *
+from .sc_nested    import *
 from .sc_math      import *
 from .sc_odict     import *
 from .sc_dataframe import *
@@ -16,3 +20,4 @@ from .sc_parallel  import *
 from .sc_fileio    import *
 from .sc_asd       import *
 from .sc_plotting  import *
+from .sc_colors    import *
