@@ -4,20 +4,21 @@ Extensions to Numpy, including finding array elements and smoothing data.
 Highlights:
     - ``sc.findinds()``: find indices of an array matching a condition
     - ``sc.findnearest()``: find nearest matching value
+    - ``sc.rolling()``: calculate rolling average
     - ``sc.smooth()``: simple smoothing of 1D or 2D arrays
-    - ``sc.smoothinterp()``: linear interpolation with smoothing
 '''
 
 import numpy as np
 import warnings
-from . import sc_utils as ut
+from . import sc_utils as scu
 
 
 ##############################################################################
-### Find and approximation functions
+#%% Find and approximation functions
 ##############################################################################
 
-__all__ = ['approx', 'safedivide', 'findinds', 'findfirst', 'findlast', 'findnearest', 'dataindex', 'getvalidinds', 'sanitize', 'getvaliddata', 'isprime']
+__all__ = ['approx', 'safedivide', 'findinds', 'findfirst', 'findlast', 'findnearest',
+           'dataindex', 'getvalidinds', 'sanitize', 'getvaliddata', 'isprime']
 
 
 def approx(val1=None, val2=None, eps=None, **kwargs):
@@ -57,14 +58,18 @@ def safedivide(numerator=None, denominator=None, default=None, eps=None, warn=Fa
     if denominator is None: denominator = 1.0
     if default     is None: default     = 0.0
 
+    # Handle types
+    if isinstance(numerator,   list): numerator   = np.array(numerator)
+    if isinstance(denominator, list): denominator = np.array(denominator)
+
     # Handle the logic
     invalid = approx(denominator, 0.0, eps=eps)
-    if ut.isnumber(denominator): # The denominator is a scalar
+    if scu.isnumber(denominator): # The denominator is a scalar
         if invalid:
             output = default
         else:
             output = numerator/denominator
-    elif ut.checktype(denominator, 'array'):
+    elif scu.checktype(denominator, 'array'):
         if not warn:
             denominator[invalid] = 1.0 # Replace invalid values with 1
         output = numerator/denominator
@@ -76,7 +81,7 @@ def safedivide(numerator=None, denominator=None, default=None, eps=None, warn=Fa
     return output
 
 
-def findinds(arr, val=None, eps=1e-6, first=False, last=False, **kwargs):
+def findinds(arr=None, val=None, eps=1e-6, first=False, last=False, die=True, **kwargs):
     '''
     Little function to find matches even if two things aren't eactly equal (eg.
     due to floats vs. ints). If one argument, find nonzero values. With two arguments,
@@ -89,23 +94,23 @@ def findinds(arr, val=None, eps=1e-6, first=False, last=False, **kwargs):
         eps (float): the precision for matching (default 1e-6, equivalent to np.isclose's atol)
         first (bool): whether to return the first matching value
         last (bool): whether to return the last matching value
+        die (bool): whether to raise an exception if first or last is true and no matches were found
         kwargs (dict): passed to np.isclose()
 
     **Examples**::
 
-        sc.findinds(rand(10)<0.5) # e.g. array([2, 4, 5, 9])
-        sc.findinds([2,3,6,3], 6) # e.g. array([2])
+        sc.findinds(rand(10)<0.5) # returns e.g. array([2, 4, 5, 9])
+        sc.findinds([2,3,6,3], 3) # returs array([1,3])
+        sc.findinds([2,3,6,3], 3, first=True) # returns 1
+
+    New in version 1.2.3: "die" argument
     '''
 
     # Handle first or last
-    if first and last:
-        raise ValueError('Can use first or last but not both')
-    elif first:
-        ind = 0
-    elif last:
-        ind = -1
-    else:
-        ind = None
+    if first and last: raise ValueError('Can use first or last but not both')
+    elif first: ind = 0
+    elif last:  ind = -1
+    else:       ind = None
 
     # Handle kwargs
     atol = kwargs.pop('atol', eps) # Ensure atol isn't specified twice
@@ -116,11 +121,11 @@ def findinds(arr, val=None, eps=1e-6, first=False, last=False, **kwargs):
         warnings.warn(warnmsg, category=DeprecationWarning, stacklevel=2)
 
     # Calculate matches
-    arr = ut.promotetoarray(arr)
+    arr = scu.promotetoarray(arr)
     if val is None: # Check for equality
         output = np.nonzero(arr) # If not, just check the truth condition
     else:
-        if ut.isstring(val):
+        if scu.isstring(val):
             output = np.nonzero(arr==val)
         try: # Standard usage, use nonzero
             output = np.nonzero(np.isclose(a=arr, b=val, atol=atol, **kwargs)) # If absolute difference between the two values is less than a certain amount
@@ -131,13 +136,20 @@ def findinds(arr, val=None, eps=1e-6, first=False, last=False, **kwargs):
                 warnings.warn(warnmsg, category=RuntimeWarning, stacklevel=2)
 
     # Process output
-    if arr.ndim == 1: # Uni-dimensional
-        output = output[0] # Return an array rather than a tuple of arrays if one-dimensional
-        if ind is not None:
-            output = output[ind] # And get the first element
-    else:
-        if ind is not None:
-            output = [output[i][ind] for i in range(arr.ndim)]
+    try:
+        if arr.ndim == 1: # Uni-dimensional
+            output = output[0] # Return an array rather than a tuple of arrays if one-dimensional
+            if ind is not None:
+                output = output[ind] # And get the first element
+        else:
+            if ind is not None:
+                output = [output[i][ind] for i in range(arr.ndim)]
+    except IndexError as E:
+        if die:
+            errormsg = 'No matching values found; use die=False to return None instead of raising an exception'
+            raise IndexError(errormsg) from E
+        else:
+            output = None
 
     return output
 
@@ -167,13 +179,13 @@ def findnearest(series=None, value=None):
 
     Version: 2017jan07
     '''
-    series = ut.promotetoarray(series)
-    if ut.isnumber(value):
+    series = scu.promotetoarray(series)
+    if scu.isnumber(value):
         output = np.argmin(abs(series-value))
     else:
         output = []
         for val in value: output.append(findnearest(series, val))
-        output = ut.promotetoarray(output)
+        output = scu.promotetoarray(output)
     return output
 
 
@@ -203,9 +215,9 @@ def getvalidinds(data=None, filterdata=None): # pragma: no cover
 
         getvalidinds([3,5,8,13], [2000, nan, nan, 2004]) # Returns array([0,3])
     '''
-    data = ut.promotetoarray(data)
+    data = scu.promotetoarray(data)
     if filterdata is None: filterdata = data # So it can work on a single input -- more or less replicates sanitize() then
-    filterdata = ut.promotetoarray(filterdata)
+    filterdata = scu.promotetoarray(filterdata)
     if filterdata.dtype=='bool': filterindices = filterdata # It's already boolean, so leave it as is
     else:                        filterindices = findinds(~np.isnan(filterdata)) # Else, assume it's nans that need to be removed
     dataindices = findinds(~np.isnan(data)) # Also check validity of data
@@ -262,7 +274,7 @@ def sanitize(data=None, returninds=False, replacenans=None, die=True, defaultval
                     sanitized = smoothinterp(newx, inds, sanitized, method=replacenans, smoothness=0) # Replace nans with interpolated values
                 else:
                     naninds = inds = np.nonzero(np.isnan(data))[0]
-                    sanitized = ut.dcp(data)
+                    sanitized = scu.dcp(data)
                     sanitized[naninds] = replacenans
             if len(sanitized)==0:
                 if defaultval is not None:
@@ -327,10 +339,10 @@ def isprime(n, verbose=False):
 
 
 ##############################################################################
-### Other functions
+#%% Other functions
 ##############################################################################
 
-__all__ += ['perturb', 'normsum', 'normalize', 'inclusiverange', 'smooth', 'smoothinterp', 'randround', 'cat']
+__all__ += ['perturb', 'normsum', 'normalize', 'inclusiverange', 'randround', 'cat']
 
 
 def perturb(n=1, span=0.5, randseed=None, normal=False):
@@ -358,7 +370,7 @@ def perturb(n=1, span=0.5, randseed=None, normal=False):
 def normsum(arr, total=None):
     '''
     Multiply a list or array by some normalizing factor so that its sum is equal
-    to the total. Formerly called sc.scaleratio().
+    to the total. Formerly called ``sc.scaleratio()``.
 
     Args:
         arr (array): array (or list) to normalize
@@ -402,16 +414,24 @@ def normalize(arr, minval=0.0, maxval=1.0):
 
 def inclusiverange(*args, **kwargs):
     '''
-    Like arange/linspace, but includes the start and stop points.
+    Like ``np.arange()``/``np.linspace()``, but includes the start and stop points.
     Accepts 0-3 args, or the kwargs start, stop, step.
 
+    In most cases, equivalent to ``np.linspace(start, stop, int((stop-start)/step)+1)``.
+
+    Args:
+        start (float): value to start at
+        stop (float): value to stop at
+        step (float): step size
+        kwargs (dict): passed to ``np.linspace()``
+
     **Examples**::
 
-        x = sc.inclusiverange(3,5,0.2)
-        x = sc.inclusiverange(stop=5)
-        x = sc.inclusiverange(6, step=2)
+        x = sc.inclusiverange(10)        # Like np.arange(11)
+        x = sc.inclusiverange(3,5,0.2)   # Like np.linspace(3, 5, int((5-3)/0.2+1))
+        x = sc.inclusiverange(stop=5)    # Like np.arange(6)
+        x = sc.inclusiverange(6, step=2) # Like np.arange(0, 7, 2)
     '''
-
     # Handle args
     if len(args)==0:
         start, stop, step = None, None, None
@@ -430,47 +450,228 @@ def inclusiverange(*args, **kwargs):
         raise ValueError('Too many arguments supplied: inclusiverange() accepts 0-3 arguments')
 
     # Handle kwargs
-    start = kwargs.get('start', start)
-    stop  = kwargs.get('stop',  stop)
-    step  = kwargs.get('step',  step)
+    start = kwargs.pop('start', start)
+    stop  = kwargs.pop('stop',  stop)
+    step  = kwargs.pop('step',  step)
 
     # Finalize defaults
     if start is None: start = 0
     if stop  is None: stop  = 1
     if step  is None: step  = 1
 
-    # OK, actually generate
-    x = np.linspace(start, stop, int(round((stop-start)/float(step))+1)) # Can't use arange since handles floating point arithmetic badly, e.g. compare arange(2000, 2020, 0.2) with arange(2000, 2020.2, 0.2)
-
+    # OK, actually generate -- can't use arange since handles floating point arithmetic badly, e.g. compare arange(2000, 2020, 0.2) with arange(2000, 2020.2, 0.2)
+    x = np.linspace(start, stop, int(round((stop-start)/float(step))+1), **kwargs)
     return x
 
 
-def smooth(data, repeats=None):
+def randround(x):
     '''
-    Very simple function to smooth a 2D array -- slow but simple and easy to use.
+    Round a float, list, or array probabilistically to the nearest integer. Works
+    for both positive and negative values.
+
+    Adapted from:
+        https://stackoverflow.com/questions/19045971/random-rounding-to-integer-in-python
+
+    Args:
+        x (int, list, arr): the floating point numbers to probabilistically convert to the nearest integer
+
+    Returns:
+        Array of integers
+
+    **Example**::
+
+        sc.randround(np.random.randn(8)) # Returns e.g. array([-1,  0,  1, -2,  2,  0,  0,  0])
+
+    New in version 1.0.0.
+    '''
+    if isinstance(x, np.ndarray):
+        output = np.array(np.floor(x+np.random.random(x.size)), dtype=int)
+    elif isinstance(x, list):
+        output = [randround(i) for i in x]
+    else:
+        output = int(np.floor(x+np.random.random()))
+    return output
+
+
+def cat(*args, axis=None, copy=False, **kwargs):
+    '''
+    Like np.concatenate(), but takes anything and returns an array. Useful for
+    e.g. appending a single number onto the beginning or end of an array.
+
+    Args:
+        args   (any):  items to concatenate into an array
+        axis   (int):  axis along which to concatenate
+        copy   (bool): whether or not to deepcopy the result
+        kwargs (dict): passed to ``np.array()``
+
+    **Examples**::
+
+        arr = sc.cat(4, np.ones(3))
+        arr = sc.cat(np.array([1,2,3]), [4,5], 6)
+        arr = sc.cat(np.random.rand(2,4), np.random.rand(2,6), axis=1)
+
+    | New in version 1.0.0.
+    | New in version 1.1.0: "copy" and keyword arguments.
+    '''
+    if not len(args):
+        return np.array([])
+    output = scu.promotetoarray(args[0])
+    for arg in args[1:]:
+        arg = scu.promotetoarray(arg)
+        output = np.concatenate((output, arg), axis=axis)
+    output = np.array(output, **kwargs)
+    if copy:
+        output = scu.dcp(output)
+    return output
+
+
+
+##############################################################################
+#%% Smoothing functions
+##############################################################################
+
+
+__all__ += ['rolling', 'convolve', 'smooth', 'smoothinterp', 'gauss1d', 'gauss2d']
+
+
+def rolling(data, window=7, operation='mean', **kwargs):
+    '''
+    Alias to Pandas' rolling() (window) method to smooth a series.
+
+    Args:
+        data (list/arr): the 1D or 2D data to be smoothed
+        window (int): the length of the window
+        operation (str): the operation to perform: 'mean' (default), 'median', 'sum', or 'none'
+        kwargs (dict): passed to pd.Series.rolling()
+
+    **Example**::
+
+        data = [5,5,5,0,0,0,0,7,7,7,7,0,0,3,3,3]
+        rolled = sc.rolling(data)
+    '''
+    import pandas as pd # Optional import
+
+    # Handle the data
+    data = np.array(data)
+    data = pd.Series(data) if data.ndim == 1 else pd.DataFrame(data)
+
+    # Perform the roll
+    roll = data.rolling(window=window, **kwargs)
+
+    # Handle output
+    if   operation in [None, 'none']: output = roll
+    elif operation == 'mean':         output = roll.mean().values
+    elif operation == 'median':       output = roll.median().values
+    elif operation == 'sum':          output = roll.sum().values
+    else:
+        errormsg = f'Operation "{operation}" not recognized; must be mean, median, sum, or none'
+        raise ValueError(errormsg)
+
+    return output
+
+
+def convolve(a, v):
+    '''
+    Like ``np.convolve()``, but always returns an array the size of the first array
+    (equivalent to mode='same'), and solves the boundary problem present in ``np.convolve()``
+    by adjusting the edges by the weight of the convolution kernel.
+
+    Args:
+        a (arr): the input array
+        v (arr): the convolution kernel
+
+    **Example**::
+
+        a = np.ones(5)
+        v = np.array([0.3, 0.5, 0.2])
+        c1 = np.convolve(a, v, mode='same') # Returns array([0.8, 1.  , 1.  , 1.  , 0.7])
+        c2 = sc.convolve(a, v)              # Returns array([1., 1., 1., 1., 1.])
+
+    | New in version 1.3.0.
+    | New in version 1.3.1: handling the case where len(a) < len(v)
+    '''
+
+    # Handle types
+    a = np.array(a)
+    v = np.array(v)
+
+    # Perform standard Numpy convolution
+    out = np.convolve(a, v, mode='same')
+
+    # Handle edge weights
+    len_a = len(a) # Length of input array
+    len_v = len(v) # Length of kernel
+    minlen = min(len_a, len_v)
+    vtot = v.sum() # Total kernel weight
+    len_lhs = minlen // 2 # Number of points to re-weight on LHS
+    len_rhs = (minlen-1) // 2 # Ditto
+    if len_lhs:
+        w_lhs = np.cumsum(v)/vtot # Cumulative sum of kernel weights on the LHS, divided by total weight
+        w_lhs = w_lhs[-1-len_lhs:-1] # Trim to the correct length
+        out[:len_lhs] = out[:len_lhs]/w_lhs # Re-weight
+    if len_rhs:
+        w_rhs = (np.cumsum(v[::-1])[::-1]/vtot) # Ditto, reversed for RHS
+        w_rhs = w_rhs[1:len_rhs+1] # Ditto
+        out[-len_rhs:] = out[-len_rhs:]/w_rhs # Ditto
+
+    # Handle the case where len(v) > len(a)
+    len_diff = max(0, len_v - len_a)
+    if len_diff:
+        lhs_trim = len_diff // 2
+        out = out[lhs_trim:lhs_trim+len_a]
+
+    return out
+
+
+def smooth(data, repeats=None, kernel=None, legacy=False):
+    '''
+    Very simple function to smooth a 1D or 2D array.
+
+    See also ``sc.gauss1d()`` for simple Gaussian smoothing.
 
     Args:
         data (arr): 1D or 2D array to smooth
-        repeats (int): number of times to apply smoothing (by default, scale with length of data)
+        repeats (int): number of times to apply smoothing (by default, scale to be 1/5th the length of data)
+        kernel (arr): the smoothing kernel to use (default: ``[0.25, 0.5, 0.25]``)
+        legacy (bool): if True, use the old (pre-1.3.0) method of calculation that doesn't correct for edge effects
 
     **Example**::
 
         data = pl.randn(5,5)
         smoothdata = sc.smooth(data)
+
+    New in version 1.3.0: Fix edge effects.
     '''
     if repeats is None:
         repeats = int(np.floor(len(data)/5))
-    output = np.array(data)
-    kernel = np.array([0.25,0.5,0.25])
-    for r in range(repeats):
-        if output.ndim == 1:
-            output = np.convolve(output, kernel, mode='same')
-        elif output.ndim == 2:
-            for i in range(output.shape[0]): output[i,:] = np.convolve(output[i,:], kernel, mode='same')
-            for j in range(output.shape[1]): output[:,j] = np.convolve(output[:,j], kernel, mode='same')
-        else: # pragma: no cover
-            errormsg = 'Simple smoothing only implemented for 1D and 2D arrays'
-            raise ValueError(errormsg)
+    if kernel is None:
+        kernel = [0.25,0.5,0.25]
+    kernel = np.array(kernel)
+    output = scu.dcp(np.array(data))
+
+    # Only convolve the kernel with itself -- equivalent to doing the full convolution multiple times
+    v = scu.dcp(kernel)
+    for r in range(repeats-1):
+        v = np.convolve(v, kernel, mode='full')
+
+    # Support legacy method of not correcting for edge effects
+    if legacy:
+        conv = np.convolve
+        kw = {'mode':'same'}
+    else:
+        conv = convolve
+        kw = {}
+
+    # Perform the convolution
+    if output.ndim == 1:
+        output = conv(output, v, **kw)
+    elif output.ndim == 2:
+        for i in range(output.shape[0]): output[i,:] = conv(output[i,:], v, **kw)
+        for j in range(output.shape[1]): output[:,j] = conv(output[:,j], v, **kw)
+    else: # pragma: no cover
+        errormsg = 'Simple smoothing only implemented for 1D and 2D arrays'
+        raise ValueError(errormsg)
+
     return output
 
 
@@ -503,9 +704,9 @@ def smoothinterp(newx=None, origx=None, origy=None, smoothness=None, growth=None
     Version: 2018jan24
     '''
     # Ensure arrays and remove NaNs
-    if ut.isnumber(newx):  newx = [newx] # Make sure it has dimension
-    if ut.isnumber(origx): origx = [origx] # Make sure it has dimension
-    if ut.isnumber(origy): origy = [origy] # Make sure it has dimension
+    if scu.isnumber(newx):  newx = [newx] # Make sure it has dimension
+    if scu.isnumber(origx): origx = [origx] # Make sure it has dimension
+    if scu.isnumber(origy): origy = [origy] # Make sure it has dimension
     newx  = np.array(newx, dtype=float)
     origx = np.array(origx, dtype=float)
     origy = np.array(origy, dtype=float)
@@ -611,62 +812,213 @@ def smoothinterp(newx=None, origx=None, origy=None, smoothness=None, growth=None
     return newy
 
 
-def randround(x):
-    '''
-    Round a float, list, or array probabilistically to the nearest integer. Works
-    for both positive and negative values.
+# For Gaussian functions -- doubles the speed to convert to 32 bit, functions faster than lambdas
+def _arr32(arr): return np.array(arr, dtype=np.float32)
+def _f32(x):     return np.float32(x)
 
-    Adapted from:
-        https://stackoverflow.com/questions/19045971/random-rounding-to-integer-in-python
+
+def gauss1d(x=None, y=None, xi=None, scale=None, use32=True):
+    '''
+    Gaussian 1D smoothing kernel.
+
+    Create smooth interpolation of input points at interpolated points. If no points
+    are supplied, use the same as the input points.
 
     Args:
-        x (int, list, arr): the floating point numbers to probabilistically convert to the nearest integer
-
-    Returns:
-        Array of integers
-
-    **Example**::
-
-        sc.randround(np.random.randn(8)) # Returns e.g. array([-1,  0,  1, -2,  2,  0,  0,  0])
-
-    New in version 1.0.0.
-    '''
-    if isinstance(x, np.ndarray):
-        output = np.array(np.floor(x+np.random.random(x.size)), dtype=int)
-    elif isinstance(x, list):
-        output = [randround(i) for i in x]
-    else:
-        output = int(np.floor(x+np.random.random()))
-    return output
-
-
-def cat(*args, axis=None, copy=False, **kwargs):
-    '''
-    Like np.concatenate(), but takes anything and returns an array. Useful for
-    e.g. appending a single number onto the beginning or end of an array.
-
-    Args:
-        args   (any):  items to concatenate into an array
-        axis   (int):  axis along which to concatenate
-        copy   (bool): whether or not to deepcopy the result
-        kwargs (dict): passed to ``np.array()``
+        x (arr): 1D list of x coordinates
+        y (arr): 1D list of y values at each of the x coordinates
+        xi (arr): 1D list of points to calculate the interpolated y
+        scale (float): how much smoothing to apply (by default, width of 5 data points)
+        use32 (bool): convert arrays to 32-bit floats (doubles speed for large arrays)
 
     **Examples**::
 
-        arr = sc.cat(4, np.ones(3))
-        arr = sc.cat(np.array([1,2,3]), [4,5], 6)
-        arr = sc.cat(np.random.rand(2,4), np.random.rand(2,6), axis=1)
+        # Setup
+        import pylab as pl
+        x = pl.rand(40)
+        y = (x-0.3)**2 + 0.2*pl.rand(40)
 
-    New in version 1.0.0.
-    New in version 1.1.0: "copy" and keyword arguments.
+        # Smooth
+        yi = sc.gauss1d(x, y)
+        yi2 = sc.gauss1d(x, y, scale=0.3)
+        xi3 = pl.linspace(0,1)
+        yi3 = sc.gauss1d(x, y, xi)
+
+        # Plot oiginal and interpolated versions
+        pl.scatter(x, y,     label='Original')
+        pl.scatter(x, yi,    label='Default smoothing')
+        pl.scatter(x, yi2,   label='More smoothing')
+        pl.scatter(xi3, yi3, label='Uniform spacing')
+        pl.show()
+
+        # Simple usage
+        sc.gauss1d(y)
+
+    New in version 1.3.0.
     '''
-    if not len(args):
-        return np.array([])
-    output = ut.promotetoarray(args[0])
-    for arg in args[1:]:
-        arg = ut.promotetoarray(arg)
-        output = np.concatenate((output, arg), axis=axis)
-    output = np.array(output, **kwargs)
-    if copy:
-        output = ut.dcp(output)
-    return output
\ No newline at end of file
+
+    # Swap inputs if x is provided but not y
+    if y is None and x is not None:
+        y,x = x,y
+    if x is None:
+        x = np.arange(len(y))
+    if xi is None:
+        xi = x
+
+    # Convert to arrays
+    try:
+        orig_dtype = y.dtype
+    except:
+        orig_dtype = np.float64
+    if use32:
+        x, y, xi, = _arr32(x), _arr32(y), _arr32(xi)
+
+    # Handle scale
+    if scale is None:
+        minmax = np.ptp(x) # Calculate the range of x
+        npts = len(x)
+        scale = 5*minmax/npts
+    scale  = _f32(scale)
+
+    def calc(xi):
+        ''' Calculate the calculation '''
+        dist = (x - xi)/scale
+        weights = np.exp(-dist**2)
+        weights = weights/np.sum(weights)
+        val = np.sum(weights*y)
+        return val
+
+    # Actual computation
+    n = len(xi)
+    yi = np.zeros(n)
+    for i in range(n):
+        yi[i] = calc(xi[i])
+
+    yi = np.array(yi, dtype=orig_dtype) # Convert back
+
+    return yi
+
+
+def gauss2d(x=None, y=None, z=None, xi=None, yi=None, scale=1.0, xscale=1.0, yscale=1.0, grid=False, use32=True):
+    '''
+    Gaussian 2D smoothing kernel.
+
+    Create smooth interpolation of input points at interpolated points. Can handle
+    either 1D or 2D inputs.
+
+    Args:
+        x (arr): 1D or 2D array of x coordinates (if None, take from z)
+        y (arr): ditto, for y
+        z (arr): 1D or 2D array of z values at each of the (x,y) points
+        xi (arr): 1D or 2D array of points to calculate the interpolated Z; if None, same as x
+        yi (arr): ditto, for y
+        scale (float): overall scale factor
+        xscale (float): ditto, just for x
+        yscale (float): ditto, just for y
+        grid (bool): if True, then return Z at a grid of (xi,yi) rather than at points
+        use32 (bool): convert arrays to 32-bit floats (doubles speed for large arrays)
+
+    **Examples**::
+
+        # Setup
+        import pylab as pl
+        x = pl.rand(40)
+        y = pl.rand(40)
+        z = 1-(x-0.5)**2 + (y-0.5)**2 # Make a saddle
+
+        # Simple usage -- only works if z is 2D
+        zi0 = sc.gauss2d(pl.rand(10,10))
+        sc.surf3d(zi0)
+
+        # Method 1 -- form grid
+        xi = pl.linspace(0,1,20)
+        yi = pl.linspace(0,1,20)
+        zi = sc.gauss2d(x, y, z, xi, yi, scale=0.1, grid=True)
+
+        # Method 2 -- use points directly
+        xi2 = pl.rand(400)
+        yi2 = pl.rand(400)
+        zi2 = sc.gauss2d(x, y, z, xi2, yi2, scale=0.1)
+
+        # Plot oiginal and interpolated versions
+        sc.scatter3d(x, y, z, c=z)
+        sc.surf3d(zi)
+        sc.scatter3d(xi2, yi2, zi2, c=zi2)
+        pl.show()
+
+    | New in version 1.3.0.
+    | New in version 1.3.1: default arguments; support for 2D inputs
+    '''
+    # Swap variables if needed
+    if z is None and x is not None:
+        z,x = x,z
+    if x is None or y is None:
+        if z.ndim != 2:
+            errormsg = f'If the x and y axes are not provided, then z must be 2D, not {z.ndim}D'
+            raise ValueError(errormsg)
+        else:
+            x = np.arange(z.shape[1]) # It's counterintuitive, but x is the 1st dimension, not the 0th
+            y = np.arange(z.shape[0])
+
+    # Handle shapes
+    if z.ndim == 2 and (x.ndim == 1) and (y.ndim == 1):
+        if (z.shape[0] != z.shape[1]) and (len(x) == z.shape[0]) and (len(y) == z.shape[1]):
+            print(f'sc.gauss2d() warning: the length of x (={len(x)}) and y (={len(y)}) match the wrong dimensions of z; transposing')
+            z = z.transpose()
+        if len(x) != z.shape[1] or len(y) != z.shape[0]:
+            errormsg = f'Shape mismatch: (y, x) = {(len(y), len(x))}, but z = {z.shape}'
+            raise ValueError(errormsg)
+
+        # If checks pass, convert to full arrays
+        x, y = np.meshgrid(x, y)
+
+    # Handle data types
+    orig_dtype = z.dtype
+    if xi is None: xi = scu.dcp(x)
+    if yi is None: yi = scu.dcp(y)
+    if use32:
+        x, y, z, xi, yi = _arr32(x), _arr32(y), _arr32(z), _arr32(xi), _arr32(yi)
+        scale, xscale, yscale = _f32(scale), _f32(xscale), _f32(yscale)
+    xsc = xscale*scale
+    ysc = yscale*scale
+
+    # Now that we have xi and yi, handle more 1D vs 2D logic
+    if xi.ndim == 1 and yi.ndim == 1 and grid:
+        xi, yi = np.meshgrid(xi, yi)
+    if xi.shape != yi.shape:
+        errormsg = f'Output arrays must have same shape, but xi = {xi.shape} and yi = {yi.shape}'
+        raise ValueError(errormsg)
+
+    # Flatten everything and check sizes
+    orig_shape = xi.shape
+    x  = x.flatten()
+    y  = y.flatten()
+    z  = z.flatten()
+    xi = xi.flatten()
+    yi = yi.flatten()
+    ni = len(xi)
+    if len(x) != len(y) != len(z):
+        errormsg = f'Input arrays do not have the same number of elements: x = {len(x)}, y = {len(y)}, z = {len(z)}'
+        raise ValueError(errormsg)
+    if len(xi) != len(yi):
+        errormsg = f'Output arrays do not have the same number of elements: xi = {len(xi)}, yi = {len(yi)}'
+        raise ValueError(errormsg)
+
+    def calc(xi, yi):
+        ''' Calculate the calculation '''
+        dist = ((x - xi)/xsc)**2 + ((y - yi)/ysc)**2
+        weights = np.exp(-dist)
+        weights = weights/np.sum(weights)
+        val = np.sum(weights*z)
+        return val
+
+    # Actual computation
+    zi = np.zeros(ni)
+    if use32: zi = _arr32(zi)
+    for i in range(ni):
+        zi[i] = calc(xi[i], yi[i])
+
+    # Convert back to original size and dtype
+    zi = np.array(zi.reshape(orig_shape), dtype=orig_dtype)
+
+    return zi
\ No newline at end of file
