@@ -5,22 +5,22 @@ Test Sciris file I/O functions.
 import os
 import pylab as pl
 import pandas as pd
-import openpyexcel
+import openpyxl
 import sciris as sc
 
-def test_legacy():
+def test_spreadsheets():
     '''
-    Preserved for completeness, but too fragile to be used in automated unit testing
-    due to reliance on openpyxl (which is not a required Sciris dependency).
+    Preserved for completeness, but fairly fragile since relies on not-well-trodden
+    Excel libraries.
     '''
 
     # Define filenames
     filedir = 'files' + os.sep
     files = sc.prettyobj()
-    files.excel  = filedir+'test.xlsx'
-    files.binary = filedir+'test.obj'
-    files.text   = filedir+'text.txt'
-    files.zip    = filedir+'test.zip'
+    files.excel  = filedir + 'test.xlsx'
+    files.binary = filedir + 'test.obj'
+    files.text   = filedir + 'text.txt'
+    files.zip    = filedir + 'test.zip'
     tidyup = True
 
     # Define the test data
@@ -51,10 +51,8 @@ def test_legacy():
     if os.path.exists(excel_path):
         sc.heading('Reading cells')
         wb = sc.Spreadsheet(filename=excel_path) # Load a sample databook to try pulling cells from
-        celltest = wb.readcells(method='xlrd', sheetname='Baseline year population inputs', cells=[[46, 2], [47, 2]]) # Grab cells using xlrd
-        celltest2 = wb.readcells(method='openpyexcel', wbargs={'data_only': True}, sheetname='Baseline year population inputs', cells=[[46, 2], [47, 2]]) # Grab cells using openpyexcel.  You have to set wbargs={'data_only': True} to pull out cached values instead of formula strings
-        print('xlrd output: %s' % celltest)
-        print('openpyxl output: %s' % celltest2)
+        celltest = wb.readcells(method='openpyxl', wbargs={'data_only': True}, sheetname='Baseline year population inputs', cells=[[46, 2], [47, 2]]) # Grab cells using openpyxl.  You have to set wbargs={'data_only': True} to pull out cached values instead of formula strings
+        print(f'openpyxl output: {celltest}')
     else:
         print(f'{excel_path} not found, skipping...')
 
@@ -62,7 +60,7 @@ def test_legacy():
     sc.heading('Loading a blobject')
     blob = sc.Blobject(files.excel)
     f = blob.tofile()
-    wb = openpyexcel.load_workbook(f)
+    wb = openpyxl.load_workbook(f)
     ws = wb.active
     ws['B7'] = 'Hi!     '
     wb.save(f)
@@ -222,7 +220,7 @@ def test_load_dump_str():
 if __name__ == '__main__':
     sc.tic()
 
-    spread = test_legacy()
+    spread = test_spreadsheets()
     json   = test_json()
     jp     = test_jsonpickle()
     string = test_load_dump_str()
